# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: simulator.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'simulator.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0fsimulator.proto\x12\x0fpulsim.api.v1\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\"\x84\x04\n\x11SimulationOptions\x12,\n\x06tstart\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12+\n\x05tstop\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12(\n\x02\x64t\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12+\n\x05\x64tmin\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12+\n\x05\x64tmax\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12,\n\x06\x61\x62stol\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12,\n\x06reltol\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12:\n\x15max_newton_iterations\x18\x08 \x01(\x0b\x32\x1b.google.protobuf.Int32Value\x12\x34\n\x0e\x64\x61mping_factor\x18\t \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12*\n\x06use_ic\x18\n \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x16\n\x0eoutput_signals\x18\x0b \x03(\t\"\x89\x01\n\x0c\x43ircuitModel\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x02 \x01(\t\x12\x12\n\nmodel_json\x18\x03 \x01(\t\x12.\n\ncreated_at\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x12\n\ncreated_by\x18\x05 \x01(\t\"Q\n\x0fModelDescriptor\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12,\n\x05model\x18\x02 \x01(\x0b\x32\x1d.pulsim.api.v1.CircuitModel\"\xac\x02\n\x11SessionDescriptor\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x10\n\x08model_id\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12.\n\x06status\x18\x04 \x01(\x0e\x32\x1e.pulsim.api.v1.SessionStatus\x12.\n\ncreated_at\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12.\n\nupdated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x16\n\x0e\x61\x63tive_signals\x18\x07 \x03(\t\x12,\n\tretention\x18\x08 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\r\n\x05owner\x18\t \x01(\t\"\x14\n\x12HealthCheckRequest\"\xd5\x01\n\x13HealthCheckResponse\x12-\n\x06status\x18\x01 \x01(\x0e\x32\x1d.pulsim.api.v1.HealthStatus\x12\x0f\n\x07version\x18\x02 \x01(\t\x12)\n\x06uptime\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x17\n\x0f\x61\x63tive_sessions\x18\x04 \x01(\r\x12\x1a\n\x12\x63ompleted_sessions\x18\x05 \x01(\r\x12\x1e\n\x16\x61uthentication_enabled\x18\x06 \x01(\x08\"\xb4\x01\n\x14\x43reateSessionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\x08model_id\x18\x02 \x01(\tH\x00\x12\x35\n\x0cinline_model\x18\x03 \x01(\x0b\x32\x1d.pulsim.api.v1.CircuitModelH\x00\x12\x33\n\x07options\x18\x04 \x01(\x0b\x32\".pulsim.api.v1.SimulationOptionsB\x0e\n\x0cmodel_source\"L\n\x15\x43reateSessionResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"\x15\n\x13ListSessionsRequest\"L\n\x14ListSessionsResponse\x12\x34\n\x08sessions\x18\x01 \x03(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"\'\n\x11GetSessionRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"I\n\x12GetSessionResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"c\n\x16StartSimulationRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x35\n\toverrides\x18\x02 \x01(\x0b\x32\".pulsim.api.v1.SimulationOptions\"N\n\x17StartSimulationResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\",\n\x16PauseSimulationRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"N\n\x17PauseSimulationResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"-\n\x17ResumeSimulationRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"O\n\x18ResumeSimulationResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"+\n\x15StopSimulationRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\"M\n\x16StopSimulationResponse\x12\x33\n\x07session\x18\x01 \x01(\x0b\x32\".pulsim.api.v1.SessionDescriptor\"\x83\x01\n\x16StreamWaveformsRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07signals\x18\x02 \x03(\t\x12\x12\n\ndecimation\x18\x03 \x01(\r\x12\x30\n\nstart_time\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\"q\n\x14WaveformStreamHeader\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07signals\x18\x02 \x03(\t\x12\x0e\n\x06tstart\x18\x03 \x01(\x01\x12\r\n\x05tstop\x18\x04 \x01(\x01\x12\x15\n\rtotal_samples\x18\x05 \x01(\x04\".\n\x0eWaveformSample\x12\x0c\n\x04time\x18\x01 \x01(\x01\x12\x0e\n\x06values\x18\x02 \x03(\x01\"e\n\x16WaveformStreamComplete\x12\x34\n\x0c\x66inal_status\x18\x01 \x01(\x0e\x32\x1e.pulsim.api.v1.SessionStatus\x12\x15\n\rerror_message\x18\x02 \x01(\t\"\xcc\x01\n\x16WaveformStreamResponse\x12\x37\n\x06header\x18\x01 \x01(\x0b\x32%.pulsim.api.v1.WaveformStreamHeaderH\x00\x12\x31\n\x06sample\x18\x02 \x01(\x0b\x32\x1f.pulsim.api.v1.WaveformSampleH\x00\x12;\n\x08\x63omplete\x18\x03 \x01(\x0b\x32\'.pulsim.api.v1.WaveformStreamCompleteH\x00\x42\t\n\x07payload\"\x99\x01\n\x10GetResultRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07signals\x18\x02 \x03(\t\x12\x30\n\nstart_time\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12.\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\"\xa4\x01\n\x0eResultMetadata\x12\x12\n\nstart_time\x18\x01 \x01(\x01\x12\x10\n\x08\x65nd_time\x18\x02 \x01(\x01\x12\x14\n\x0csample_count\x18\x03 \x01(\x04\x12\x0f\n\x07signals\x18\x04 \x03(\t\x12.\n\x06status\x18\x05 \x01(\x0e\x32\x1e.pulsim.api.v1.SessionStatus\x12\x15\n\rerror_message\x18\x06 \x01(\t\"F\n\x11GetResultResponse\x12\x31\n\x08metadata\x18\x01 \x01(\x0b\x32\x1f.pulsim.api.v1.ResultMetadata\"\xd7\x01\n\x15\x44ownloadResultRequest\x12\x12\n\nsession_id\x18\x01 \x01(\t\x12\x0f\n\x07signals\x18\x02 \x03(\t\x12\x30\n\nstart_time\x18\x03 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12.\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.DoubleValue\x12\x37\n\x06\x66ormat\x18\x05 \x01(\x0e\x32\'.pulsim.api.v1.SimulationResultFormat\"c\n\x16\x44ownloadResultResponse\x12\r\n\x05\x63hunk\x18\x01 \x01(\x0c\x12\x12\n\nlast_chunk\x18\x02 \x01(\x08\x12\x14\n\x0c\x63ontent_type\x18\x03 \x01(\t\x12\x10\n\x08\x66ilename\x18\x04 \x01(\t\"B\n\x12UploadModelRequest\x12,\n\x05model\x18\x01 \x01(\x0b\x32\x1d.pulsim.api.v1.CircuitModel\"F\n\x13UploadModelResponse\x12/\n\x05model\x18\x01 \x01(\x0b\x32 .pulsim.api.v1.ModelDescriptor\"\x13\n\x11ListModelsRequest\"F\n\x12ListModelsResponse\x12\x30\n\x06models\x18\x01 \x03(\x0b\x32 .pulsim.api.v1.ModelDescriptor\"#\n\x0fGetModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\"C\n\x10GetModelResponse\x12/\n\x05model\x18\x01 \x01(\x0b\x32 .pulsim.api.v1.ModelDescriptor\"5\n\x12\x44\x65leteModelRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\r\n\x05\x66orce\x18\x02 \x01(\x08\"&\n\x13\x44\x65leteModelResponse\x12\x0f\n\x07\x64\x65leted\x18\x01 \x01(\x08\"_\n\x0eSweepParameter\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05start\x18\x02 \x01(\x01\x12\x0c\n\x04stop\x18\x03 \x01(\x01\x12\r\n\x05steps\x18\x04 \x01(\r\x12\x13\n\x0blogarithmic\x18\x05 \x01(\x08\"\xa6\x01\n\x12\x43reateSweepRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x38\n\x0c\x62\x61se_options\x18\x02 \x01(\x0b\x32\".pulsim.api.v1.SimulationOptions\x12\x33\n\nparameters\x18\x03 \x03(\x0b\x32\x1f.pulsim.api.v1.SweepParameter\x12\x0f\n\x07signals\x18\x04 \x03(\t\"\'\n\x13\x43reateSweepResponse\x12\x10\n\x08sweep_id\x18\x01 \x01(\t\"5\n\x0fRunSweepRequest\x12\x10\n\x08sweep_id\x18\x01 \x01(\t\x12\x10\n\x08parallel\x18\x02 \x01(\x08\"@\n\x10RunSweepResponse\x12,\n\x06status\x18\x01 \x01(\x0e\x32\x1c.pulsim.api.v1.SweepStatus\"*\n\x16GetSweepResultsRequest\x12\x10\n\x08sweep_id\x18\x01 \x01(\t\"-\n\x0eParameterValue\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01\"\x8b\x01\n\x0bSweepResult\x12\x33\n\nparameters\x18\x01 \x03(\x0b\x32\x1f.pulsim.api.v1.ParameterValue\x12\x31\n\x08metadata\x18\x02 \x01(\x0b\x32\x1f.pulsim.api.v1.ResultMetadata\x12\x14\n\x0c\x66inal_values\x18\x03 \x03(\x01\"v\n\x17GetSweepResultsResponse\x12,\n\x06status\x18\x01 \x01(\x0e\x32\x1c.pulsim.api.v1.SweepStatus\x12-\n\x07results\x18\x02 \x03(\x0b\x32\x1c.pulsim.api.v1.SweepResult*x\n\x0cHealthStatus\x12\x1d\n\x19HEALTH_STATUS_UNSPECIFIED\x10\x00\x12\x14\n\x10HEALTH_STATUS_OK\x10\x01\x12\x1a\n\x16HEALTH_STATUS_DEGRADED\x10\x02\x12\x17\n\x13HEALTH_STATUS_ERROR\x10\x03*\x90\x02\n\rSessionStatus\x12\x1e\n\x1aSESSION_STATUS_UNSPECIFIED\x10\x00\x12\x1a\n\x16SESSION_STATUS_CREATED\x10\x01\x12\x1d\n\x19SESSION_STATUS_VALIDATING\x10\x02\x12\x18\n\x14SESSION_STATUS_READY\x10\x03\x12\x1a\n\x16SESSION_STATUS_RUNNING\x10\x04\x12\x19\n\x15SESSION_STATUS_PAUSED\x10\x05\x12\x1c\n\x18SESSION_STATUS_COMPLETED\x10\x06\x12\x1a\n\x16SESSION_STATUS_STOPPED\x10\x07\x12\x19\n\x15SESSION_STATUS_FAILED\x10\x08*\xad\x01\n\x16SimulationResultFormat\x12(\n$SIMULATION_RESULT_FORMAT_UNSPECIFIED\x10\x00\x12 \n\x1cSIMULATION_RESULT_FORMAT_CSV\x10\x01\x12!\n\x1dSIMULATION_RESULT_FORMAT_HDF5\x10\x02\x12$\n SIMULATION_RESULT_FORMAT_PARQUET\x10\x03*\xb0\x01\n\x0bSweepStatus\x12\x1c\n\x18SWEEP_STATUS_UNSPECIFIED\x10\x00\x12\x18\n\x14SWEEP_STATUS_PENDING\x10\x01\x12\x18\n\x14SWEEP_STATUS_RUNNING\x10\x02\x12\x1a\n\x16SWEEP_STATUS_COMPLETED\x10\x03\x12\x17\n\x13SWEEP_STATUS_FAILED\x10\x04\x12\x1a\n\x16SWEEP_STATUS_CANCELLED\x10\x05\x32\xa5\r\n\x10SimulatorService\x12X\n\x0bHealthCheck\x12#.pulsim.api.v1.HealthCheckRequest\x1a$.pulsim.api.v1.HealthCheckResponse\x12^\n\rCreateSession\x12%.pulsim.api.v1.CreateSessionRequest\x1a&.pulsim.api.v1.CreateSessionResponse\x12[\n\x0cListSessions\x12$.pulsim.api.v1.ListSessionsRequest\x1a%.pulsim.api.v1.ListSessionsResponse\x12U\n\nGetSession\x12\".pulsim.api.v1.GetSessionRequest\x1a#.pulsim.api.v1.GetSessionResponse\x12\x64\n\x0fStartSimulation\x12\'.pulsim.api.v1.StartSimulationRequest\x1a(.pulsim.api.v1.StartSimulationResponse\x12\x64\n\x0fPauseSimulation\x12\'.pulsim.api.v1.PauseSimulationRequest\x1a(.pulsim.api.v1.PauseSimulationResponse\x12g\n\x10ResumeSimulation\x12(.pulsim.api.v1.ResumeSimulationRequest\x1a).pulsim.api.v1.ResumeSimulationResponse\x12\x61\n\x0eStopSimulation\x12&.pulsim.api.v1.StopSimulationRequest\x1a\'.pulsim.api.v1.StopSimulationResponse\x12\x65\n\x0fStreamWaveforms\x12\'.pulsim.api.v1.StreamWaveformsRequest\x1a\'.pulsim.api.v1.WaveformStreamResponse0\x01\x12R\n\tGetResult\x12!.pulsim.api.v1.GetResultRequest\x1a\".pulsim.api.v1.GetResultResponse\x12\x63\n\x0e\x44ownloadResult\x12&.pulsim.api.v1.DownloadResultRequest\x1a\'.pulsim.api.v1.DownloadResultResponse0\x01\x12X\n\x0bUploadModel\x12#.pulsim.api.v1.UploadModelRequest\x1a$.pulsim.api.v1.UploadModelResponse\x12U\n\nListModels\x12\".pulsim.api.v1.ListModelsRequest\x1a#.pulsim.api.v1.ListModelsResponse\x12O\n\x08GetModel\x12 .pulsim.api.v1.GetModelRequest\x1a!.pulsim.api.v1.GetModelResponse\x12X\n\x0b\x44\x65leteModel\x12#.pulsim.api.v1.DeleteModelRequest\x1a$.pulsim.api.v1.DeleteModelResponse\x12X\n\x0b\x43reateSweep\x12#.pulsim.api.v1.CreateSweepRequest\x1a$.pulsim.api.v1.CreateSweepResponse\x12O\n\x08RunSweep\x12 .pulsim.api.v1.RunSweepRequest\x1a!.pulsim.api.v1.RunSweepResponse\x12\x64\n\x0fGetSweepResults\x12\'.pulsim.api.v1.GetSweepResultsRequest\x1a(.pulsim.api.v1.GetSweepResultsResponseBc\n\x13\x63om.pulsim.api.v1B\x16PulsimSimulatorProtoP\x01Z github.com/pulsim/api/v1;apiv1\xaa\x02\x0fPulsim.Api.V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'simulator_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023com.pulsim.api.v1B\026PulsimSimulatorProtoP\001Z github.com/pulsim/api/v1;apiv1\252\002\017Pulsim.Api.V1'
  _globals['_HEALTHSTATUS']._serialized_start=5018
  _globals['_HEALTHSTATUS']._serialized_end=5138
  _globals['_SESSIONSTATUS']._serialized_start=5141
  _globals['_SESSIONSTATUS']._serialized_end=5413
  _globals['_SIMULATIONRESULTFORMAT']._serialized_start=5416
  _globals['_SIMULATIONRESULTFORMAT']._serialized_end=5589
  _globals['_SWEEPSTATUS']._serialized_start=5592
  _globals['_SWEEPSTATUS']._serialized_end=5768
  _globals['_SIMULATIONOPTIONS']._serialized_start=163
  _globals['_SIMULATIONOPTIONS']._serialized_end=679
  _globals['_CIRCUITMODEL']._serialized_start=682
  _globals['_CIRCUITMODEL']._serialized_end=819
  _globals['_MODELDESCRIPTOR']._serialized_start=821
  _globals['_MODELDESCRIPTOR']._serialized_end=902
  _globals['_SESSIONDESCRIPTOR']._serialized_start=905
  _globals['_SESSIONDESCRIPTOR']._serialized_end=1205
  _globals['_HEALTHCHECKREQUEST']._serialized_start=1207
  _globals['_HEALTHCHECKREQUEST']._serialized_end=1227
  _globals['_HEALTHCHECKRESPONSE']._serialized_start=1230
  _globals['_HEALTHCHECKRESPONSE']._serialized_end=1443
  _globals['_CREATESESSIONREQUEST']._serialized_start=1446
  _globals['_CREATESESSIONREQUEST']._serialized_end=1626
  _globals['_CREATESESSIONRESPONSE']._serialized_start=1628
  _globals['_CREATESESSIONRESPONSE']._serialized_end=1704
  _globals['_LISTSESSIONSREQUEST']._serialized_start=1706
  _globals['_LISTSESSIONSREQUEST']._serialized_end=1727
  _globals['_LISTSESSIONSRESPONSE']._serialized_start=1729
  _globals['_LISTSESSIONSRESPONSE']._serialized_end=1805
  _globals['_GETSESSIONREQUEST']._serialized_start=1807
  _globals['_GETSESSIONREQUEST']._serialized_end=1846
  _globals['_GETSESSIONRESPONSE']._serialized_start=1848
  _globals['_GETSESSIONRESPONSE']._serialized_end=1921
  _globals['_STARTSIMULATIONREQUEST']._serialized_start=1923
  _globals['_STARTSIMULATIONREQUEST']._serialized_end=2022
  _globals['_STARTSIMULATIONRESPONSE']._serialized_start=2024
  _globals['_STARTSIMULATIONRESPONSE']._serialized_end=2102
  _globals['_PAUSESIMULATIONREQUEST']._serialized_start=2104
  _globals['_PAUSESIMULATIONREQUEST']._serialized_end=2148
  _globals['_PAUSESIMULATIONRESPONSE']._serialized_start=2150
  _globals['_PAUSESIMULATIONRESPONSE']._serialized_end=2228
  _globals['_RESUMESIMULATIONREQUEST']._serialized_start=2230
  _globals['_RESUMESIMULATIONREQUEST']._serialized_end=2275
  _globals['_RESUMESIMULATIONRESPONSE']._serialized_start=2277
  _globals['_RESUMESIMULATIONRESPONSE']._serialized_end=2356
  _globals['_STOPSIMULATIONREQUEST']._serialized_start=2358
  _globals['_STOPSIMULATIONREQUEST']._serialized_end=2401
  _globals['_STOPSIMULATIONRESPONSE']._serialized_start=2403
  _globals['_STOPSIMULATIONRESPONSE']._serialized_end=2480
  _globals['_STREAMWAVEFORMSREQUEST']._serialized_start=2483
  _globals['_STREAMWAVEFORMSREQUEST']._serialized_end=2614
  _globals['_WAVEFORMSTREAMHEADER']._serialized_start=2616
  _globals['_WAVEFORMSTREAMHEADER']._serialized_end=2729
  _globals['_WAVEFORMSAMPLE']._serialized_start=2731
  _globals['_WAVEFORMSAMPLE']._serialized_end=2777
  _globals['_WAVEFORMSTREAMCOMPLETE']._serialized_start=2779
  _globals['_WAVEFORMSTREAMCOMPLETE']._serialized_end=2880
  _globals['_WAVEFORMSTREAMRESPONSE']._serialized_start=2883
  _globals['_WAVEFORMSTREAMRESPONSE']._serialized_end=3087
  _globals['_GETRESULTREQUEST']._serialized_start=3090
  _globals['_GETRESULTREQUEST']._serialized_end=3243
  _globals['_RESULTMETADATA']._serialized_start=3246
  _globals['_RESULTMETADATA']._serialized_end=3410
  _globals['_GETRESULTRESPONSE']._serialized_start=3412
  _globals['_GETRESULTRESPONSE']._serialized_end=3482
  _globals['_DOWNLOADRESULTREQUEST']._serialized_start=3485
  _globals['_DOWNLOADRESULTREQUEST']._serialized_end=3700
  _globals['_DOWNLOADRESULTRESPONSE']._serialized_start=3702
  _globals['_DOWNLOADRESULTRESPONSE']._serialized_end=3801
  _globals['_UPLOADMODELREQUEST']._serialized_start=3803
  _globals['_UPLOADMODELREQUEST']._serialized_end=3869
  _globals['_UPLOADMODELRESPONSE']._serialized_start=3871
  _globals['_UPLOADMODELRESPONSE']._serialized_end=3941
  _globals['_LISTMODELSREQUEST']._serialized_start=3943
  _globals['_LISTMODELSREQUEST']._serialized_end=3962
  _globals['_LISTMODELSRESPONSE']._serialized_start=3964
  _globals['_LISTMODELSRESPONSE']._serialized_end=4034
  _globals['_GETMODELREQUEST']._serialized_start=4036
  _globals['_GETMODELREQUEST']._serialized_end=4071
  _globals['_GETMODELRESPONSE']._serialized_start=4073
  _globals['_GETMODELRESPONSE']._serialized_end=4140
  _globals['_DELETEMODELREQUEST']._serialized_start=4142
  _globals['_DELETEMODELREQUEST']._serialized_end=4195
  _globals['_DELETEMODELRESPONSE']._serialized_start=4197
  _globals['_DELETEMODELRESPONSE']._serialized_end=4235
  _globals['_SWEEPPARAMETER']._serialized_start=4237
  _globals['_SWEEPPARAMETER']._serialized_end=4332
  _globals['_CREATESWEEPREQUEST']._serialized_start=4335
  _globals['_CREATESWEEPREQUEST']._serialized_end=4501
  _globals['_CREATESWEEPRESPONSE']._serialized_start=4503
  _globals['_CREATESWEEPRESPONSE']._serialized_end=4542
  _globals['_RUNSWEEPREQUEST']._serialized_start=4544
  _globals['_RUNSWEEPREQUEST']._serialized_end=4597
  _globals['_RUNSWEEPRESPONSE']._serialized_start=4599
  _globals['_RUNSWEEPRESPONSE']._serialized_end=4663
  _globals['_GETSWEEPRESULTSREQUEST']._serialized_start=4665
  _globals['_GETSWEEPRESULTSREQUEST']._serialized_end=4707
  _globals['_PARAMETERVALUE']._serialized_start=4709
  _globals['_PARAMETERVALUE']._serialized_end=4754
  _globals['_SWEEPRESULT']._serialized_start=4757
  _globals['_SWEEPRESULT']._serialized_end=4896
  _globals['_GETSWEEPRESULTSRESPONSE']._serialized_start=4898
  _globals['_GETSWEEPRESULTSRESPONSE']._serialized_end=5016
  _globals['_SIMULATORSERVICE']._serialized_start=5771
  _globals['_SIMULATORSERVICE']._serialized_end=7472
# @@protoc_insertion_point(module_scope)
